{$R+}

(* ARRESTIM.PAS

This Turbo Pascal program for standard MS-DOS estimates the internal
rate of return by implementing the Average Accountant's Rate of
Return method.

*)

program ARREstimProgram;  { IRR estimation by average ARR }

uses SMUCTPU;  { A unit of common routines }

const progDesc = 'A Turbo Pascal program to estimate ' +
                 'Average ARR method';
      progName = 'ARRESTIM';
      progDate = 'Sat 12-Oct-96';
      progVers = '1.0';
      maxYears = 50;

type realVectorType = array [1..maxYears] of real;

var p,  { accountant's profit }
    v   { book value }
     : realVectorType;

var n   { number of years }
     : integer;

var irrEstimate { the result }
     : real;

(* Read and display the input data *)
procedure READDATA (var p, v : realVectorType; var n : integer);
var s : string;   { read line as string }
    k : integer;  { error index }
    j : integer;  { position of the comment initiator '#' }
    year : integer;
    filePointer : text;
    flag : boolean;
begin
  {}
  {... open the input file ...}
  if ParamCount > 0 then begin
    if not FILEXIST (ParamStr(1)) then begin
      writeln;
      writeln ('File ', ParamStr(1), ' not found');
      halt;
    end; {if}
    assign (filePointer, ParamStr(1));
  end
  else begin
    writeln;
    writeln ('Usage: ', progName, ' InputFileName');
    halt;
  end; {endif}
  reset (filePointer);
  {}
  {... read the annual data from the data file ...}
  n := 1;
  flag := true;
  while not eof(filePointer) do begin
    readln (filePointer, s);
    s := TRAILFN(s);
    if s = 'end' then break;
    j := Pos('#',s);
    if (Length(s) > 0) and ((j = 0) or (j > 1)) then begin
      if j > 1 then s := TRAILFN (Copy (s, 1, j-1));
      Val (PARSERFN(s,1), year, k);
      Val (PARSERFN(s,5), p[n], k);
      Val (PARSERFN(s,6), v[n], k);
      if flag then begin
        writeln ('Year   Operating income   Book value');
        flag := false;
      end; {if}
      writeln (year : 4, p[n]:19:4, v[n]:13:4);
      Inc (n);
    end {if}
      else writeln (s);
  end; {while}
  {}
  close (filePointer);
  n := n - 1;
  writeln ('Number of years: ', n);
end;  (* readdata *)

(* Solve Kay's a estimate of the internal rate of return,
   secant method *)
function IRRESTFN (p, v : realVectorType;
                   n : integer) : real;
const ff = 12; dd = 4;
var i : integer;
    sum : real;
begin
  sum := 0;
  for i := 1 to n do begin
    write (i:4, p[i]:ff:dd, v[i]:ff:dd);
    if i = 1 then writeln
    else begin
      if (abs(v[i-1]) - 1.0E-7) < 0 then begin
        writeln ('Algorith failed: Division by near zero');
        halt;
      end
      else writeln (p[i]/v[i-1]:ff:dd);
    end; {else}
    if i > 1 then sum := sum + p[i]/v[i-1];
  end; {for i}
  irrestfn := sum / (n-1.0);
end;  (* irrestfn *)

(* Main program *)
begin
  FileMode := 0;
  HEADER (progName, progDesc, progVers, progDate, '');
  READDATA (p, v, n);
  irrEstimate := IRRESTFN (p, v, n);
  writeln;
  writeln ('E(ARR) estimate of the internal rate of return = ',
            100.0*irrEstimate:0:4, '%');
end.  (* ARREstimProgram *)
